using UnityEngine;
using System.Collections;

[RequireComponent (typeof (CharacterController))]

// This is an updated example of a platformer using the character controller.
// While the characterController has some limitations, it does simplify a number of things. 
// Most importantly, you can 
public class AvatarScript : MonoBehaviour {
	// movement property references
    public float speed = 6.0F;
    public float jumpSpeed = 8.0F;
	public float gravity = 10.0F;

	// our movement vector
	private Vector3 moveDirection = Vector3.zero;

	// Animation references:  if you want to control the kids and do something with them
	GameObject head;
	GameObject body;
	
	// some state for fiddling with our jump animation
	float initY;
	float jumpTime;

	// number of jumps	
	int jumps = 0;
	
	// state machine:  not using it yet
	private ImmediateStateMachine stateMachine = new ImmediateStateMachine ();

	// our Character Controller
	CharacterController controller;

	// another way of doing some small debugging;  just put some GUI text boxes up and fill them in as needed
	GUIText debugText;
	string dbgTxt = "";

	// public access to increment boost count
	void AddJump(){
		jumps += 1;
		GUIManager.SetJumps(jumps);
	}

	void Start (){
		controller = gameObject.GetComponent<CharacterController>(); 
		debugText = GameObject.Find("debugText").guiText;
		
		GameEventManager.GameStart += GameStart;
		GameEventManager.GameOver += GameOver;
		
     	// Assign body parts to variables;  
		// -> could also have these as properties you set in editor
		// -> could also have used Transform.Find to only search in the children of this object
	 	head = GameObject.Find ("Head");
     	body = GameObject.Find ("Body");
		
		gameObject.SetActive(false);	
	}
	
	void GameStart() {
		initY = 0;
		jumpTime = 0;
		gameObject.SetActive(true);	
	}
	
	void GameOver() {
		// won't be called
		gameObject.SetActive(false);
	}
	
	void Update() {
		dbgTxt = "";
		//dbgTxt = "velocity: " + controller.velocity + " ";
		dbgTxt = "flags: " + controller.collisionFlags + " ";
		stateMachine.Execute();
					
		// we only do input if on the ground. If you want to do left/right movement in the air, you 
		// need to deal with it differently because you can't just reset the vector (you need to 
		// add the input to the vector, as you do gravity)
        if (controller.isGrounded) {
            moveDirection = new Vector3(Input.GetAxis("Horizontal"), -0.1f, 0f);
            moveDirection = transform.TransformDirection(moveDirection);
            moveDirection *= speed;
			
			// move up off the ground by adding an upward impulse
			if (Input.GetButton("Jump")){
				AddJump();
                moveDirection.y = jumpSpeed;
				body.transform.Rotate(new Vector3(0,0,0));    
				initY = transform.position.y;
				jumpTime = Time.time + 0.1f;
			}
        } else {
			if ((controller.collisionFlags & CollisionFlags.CollidedSides) != 0)
			{
				// if we are not grounded, but colliding with the sides, we can wall jump
				if (Input.GetButton("Jump")){
					AddJump();
        	        moveDirection.y = jumpSpeed / 3f;
					if (moveDirection.x > 0)
						moveDirection.x = -(jumpSpeed / 2f);
					else
						moveDirection.x = (jumpSpeed / 2f);
					body.transform.Rotate(new Vector3(0,0,0));    
					initY = transform.position.y;
					jumpTime = Time.time;
				}
			}
			Vector3 vec = new Vector3(Input.GetAxis("Horizontal") + moveDirection.x * speed * -0.001f, 0f, 0f);
			moveDirection += transform.TransformDirection(vec) * speed * 0.01f;
		}
		
		// if we are moving up (jumping) do silly things with the head object.  If we are moving down,
		// do something different, yet also silly.  When we are walking, do something different, yet
		// just as silly
		if (controller.isGrounded) 
		{ 
			head.transform.localPosition = new Vector3 (0F,0f,0F);
			if (controller.velocity.x > 0 ) { 	
				body.transform.localRotation = Quaternion.Euler (0,0,-10);
			} else if (controller.velocity.x < 0) {
				body.transform.localRotation = Quaternion.Euler (0,0,10);
			} else {
				body.transform.localRotation = Quaternion.Euler (0,0,0);
			}
		} else {
			if(controller.velocity.y > 1) {
				// Jumping 
				head.transform.localPosition = new Vector3(0F,1f,0F);
				if (jumpTime > Time.time) {
					body.transform.localPosition = new Vector3(0F, (initY - transform.position.y), 0f);
				} else {
					body.transform.localPosition = new Vector3(0F, 0f, 0f);
				}
				body.transform.localRotation = Quaternion.Euler (0,0,0);
			} else if(controller.velocity.y < -0.35) {
				// Falling
				body.transform.localPosition = new Vector3(0F, 0f, 0f);
				if (moveDirection.x > 0.01f) {
					head.transform.localPosition = new Vector3(1f,0f,0F);
					body.transform.localRotation = Quaternion.Euler (0,0,20f);
				} else if (moveDirection.x < -0.01f) {
					head.transform.localPosition = new Vector3(-1f,0f,0F);
					body.transform.localRotation = Quaternion.Euler (0,0,-20f);
				} else {
					head.transform.localPosition = new Vector3(0f,0f,0F);
					body.transform.localRotation = Quaternion.Euler (0,0,0f);
				}
		    } 
		}

		// if we're on the ground, and are "inside" whatever we are on, move "almost" out.  If we are 
		// in the air, apply some gravity
		if (!controller.isGrounded) 
		{
			if (moveDirection.y < 0 && moveDirection.y > -1.0)
				moveDirection.y -= gravity * Time.deltaTime * 10f;
			else
				moveDirection.y -= gravity * Time.deltaTime;
		} 
		
		// a little nudge to keep our character at z=0, since you can't lock the movement of a character controller 
		// like you can with a rigidbody, but the physics engine may push it off the z=0 plane.
		moveDirection.z = -transform.position.z;
		
		// move, and adjust speeds based on collisions.  Need to do this to avoid the horrible sliding motions
		// that the Controller does otherwise
		CollisionFlags flags = controller.Move(moveDirection * Time.deltaTime);	
		
        if ((flags & CollisionFlags.CollidedSides) != 0)
		{
			// keep it moving the same direction but at a VERY small rate (so the collision stays consistently on if the player
			// is pushing in that direction)
			moveDirection.x /= 10.0f;
			moveDirection.y /= 2.0f;  // slow down the vertical movement 
		}
		
		if ((flags & CollisionFlags.CollidedAbove) != 0)
		{
			// start moving down immediately by a little.  Ouch, my head!
			moveDirection.y = -gravity * Time.deltaTime * 2f;
			moveDirection.x /= 1.15f;  // slow down sideways movement
		}
				
		debugText.text = dbgTxt;
    }
	
    void OnControllerColliderHit(ControllerColliderHit hit) {
		// here you would put things related to specific collisions
    }
}